﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Dynamic;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using Infragistics.Controls.Editors;
using Infragistics.Controls.Editors.Primitives;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;

namespace ReproduceInfragisticsIssue
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private String strCharTyped = null;

        private void OptionsDataGrid_OnKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers.HasFlag(ModifierKeys.Control))
            {
                return;
            }

            XamDataGrid oGrid = sender as XamDataGrid;
            if (oGrid.ActiveCell != null)
            {
                if (e.Key == Key.Enter)
                {
                    if (oGrid.ActiveRecord.IsSpecialRecord)
                        oGrid.ExecuteCommand(DataPresenterCommands.EndEditModeAndCommitRecord);
                }
            }
        }

        private void OptionsDataGrid_OnFieldLayoutInitialized(object sender, FieldLayoutInitializedEventArgs e)
        {
            var dg = sender as XamDataGrid;
            var first = dg.DataSource.Cast<Object>().FirstOrDefault() as DynamicObject;
            if (first == null)
                return;
            List<String> names = first.GetDynamicMemberNames().ToList();

            dg.AddHandler(DataPresenterBase.EditModeStartedEvent, new EventHandler<EditModeStartedEventArgs>(LoadMultiColumnComboEditor));

            e.FieldLayout.Fields.Clear();

            foreach (string name in names)
            {
                if (name == "Selected")
                    continue;

                Field field = new Field
                {
                    Name = name,
                    Label = name,
                    BindingType = BindingType.UseAlternateBinding,
                    AlternateBinding = new Binding(String.Format("[{0}]", name.Replace("^", "^^"))) { Mode = BindingMode.TwoWay },
                    Width = new FieldLength(1, FieldLengthUnitType.Auto),
                    DataType = typeof(String)
                };

                field.Settings.LabelMinWidth = 100;
                field.Settings.CellClickAction = CellClickAction.SelectCell;

                if (name != "Label")
                {
                    field.EditorStyle = this.FindResource("MultiComboTextStyle") as Style;
                }

                e.FieldLayout.Fields.Add(field);
            }
        }

        private void LoadMultiColumnComboEditor(object sender, EditModeStartedEventArgs e)
        {
            if (e.Cell.Field.Index == 0)
            {
                return;
            }

            MyViewModel svm = this.DataContext as MyViewModel;
            svm.DropDownDisplayMemberPath = "Value";
            svm.DropdownValues = new ObservableCollection<LevelInfoItem>()
            {
                new LevelInfoItem() { Label = "Option 1", Value = "1" },
                new LevelInfoItem() { Label = "Option 2", Value = "2" },
                new LevelInfoItem() { Label = "Option 3", Value = "3" },
                new LevelInfoItem() { Label = "Option 4", Value = "4" },
                new LevelInfoItem() { Label = "Option 5", Value = "5" },
                new LevelInfoItem() { Label = "Option 6", Value = "6" },
            };

            strCharTyped = strCharTyped ?? (string)e.Cell.Value;
        }

        private void XamMultiColumnComboEditor_Loaded(object sender, RoutedEventArgs e)
        {
            SpecializedTextBox txt = Utilities.GetDescendantFromType(sender as DependencyObject, typeof(SpecializedTextBox), false) as SpecializedTextBox;
            XamMultiColumnComboEditor editor = (XamMultiColumnComboEditor)sender;
            editor.IsDropDownOpen = true;
            if (strCharTyped == null)
            {
                txt.SelectAll();
            }
            else if (strCharTyped.Length > 1)
            {
                txt.Text = strCharTyped;
                txt.SelectAll();
            }
            else
            {
                txt.Text = strCharTyped;
                txt.CaretIndex = txt.Text.Length;
            }
            txt.Focus();
        }

        private void XamMultiColumnComboEditor_OnSelectionChanged(object sender, Infragistics.Controls.Editors.SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                CellValuePresenter cvp = (CellValuePresenter)Utilities.GetAncestorFromType(sender as DependencyObject, typeof(CellValuePresenter), false);
                LevelInfoItem item = e.AddedItems[0] as LevelInfoItem;
                cvp.Record.Cells[cvp.Field].Value = item.Value;
            }
        }

        private Boolean bCloseComboEditor = false;

        private void XamMultiColumnComboEditor_OnDropDownOpening(object sender, CancelEventArgs e)
        {
            XamMultiColumnComboEditor editor = (XamMultiColumnComboEditor)sender;
            // this is a hack because there is a bugg in the control
            editor.MinDropDownWidth = 500;
            bCloseComboEditor = true;
        }

        private void XamMultiColumnComboEditor_OnDropDownClosing(object sender, CancelEventArgs e)
        {
            if (bCloseComboEditor)
            {
                MyDataGrid.ExecuteCommand(DataPresenterCommands.EndEditModeAndCommitRecord);
                bCloseComboEditor = false;
            }
        }

        private void XamMultiColumnComboEditor_OnPreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Tab)
            {
                MyViewModel oVM = (MyViewModel)this.DataContext;
                if (oVM.DropDownDisplayMemberPath != "Label")
                {
                    return;
                }

                XamMultiColumnComboEditor editor = sender as XamMultiColumnComboEditor;
                MyDataGrid.ExecuteCommand(DataPresenterCommands.EndEditModeAndCommitRecord);

                e.Handled = true;

                editor.IsDropDownOpen = false;

                MyDataGrid.ExecuteCommand(DataPresenterCommands.CellNextByTab);
            }
            else if (e.Key == Key.Escape)
            {
                MyDataGrid.ExecuteCommand(DataPresenterCommands.EndEditModeAndDiscardChanges);
            }
        }

    }
}
